;**********************************************************************
;======================================================================
; Solar Striker Project
;
; Differential Scrolling subroutines
; NeXO (Benoit SCHERRER) / Etan21 (Brian TRIBONDEAU)
;======================================================================
;**********************************************************************



;----------------------------------------------------------------------
;////////////// VARIABLES /////////////////////////////////////////////
;----------------------------------------------------------------------

scroll_vars:

bg_vars:
background_offset	dc.w	0
background_addr	dc.l	0
background_tiles	dc.l	0
background_timer	dc.w	0
background_itimer	dc.w	0


fg_vars:
foreground_offset	dc.w	0
foreground_addr	dc.l	0
foreground_tiles	dc.l	0
foreground_timer	dc.w	0
foreground_itimer	dc.w	0

fg_enabled		dc.w	0

sfg_vars:
sforeground_offset	dc.w	0
sforeground_addr	dc.l	0
sforeground_tiles	dc.l	0
sforeground_timer	dc.w	0
sforeground_itimer	dc.w	0

;----------------------------------------------------------------------
;////////////// MACROS ////////////////////////////////////////////////
;----------------------------------------------------------------------

; Use : Unmasked32Pix [Src]
;--------------------------
Unmasked32Pix	MACRO
			move.l	(\1)+,(a0)+
			move.l	(\1)+,(a1)+
		ENDM


; Use : Masked32Pix [Src]
;------------------------
Masked32Pix	MACRO
			move.l	(\1)+,d0
			move.l	(\1)+,d1
			move.l	(\1)+,d2
			move.l	(\1)+,d5

			not.l	d0
			and.l	d0,(a0)			;Use Mask
			or.l	d1,(a0)+		;Put GFX

			not.l	d2
			and.l	d2,(a1)			;Use Mask
			or.l	d5,(a1)+		;Put GFX
		ENDM	


;----------------------------------------------------------------------
;////////////// CONSTANTS /////////////////////////////////////////////
;----------------------------------------------------------------------
TILE_HEIGHT	EQU	32


;----------------------------------------------------------------------
;////////////// SUBROUTINES ///////////////////////////////////////////
;----------------------------------------------------------------------


;----------------------------------------------------------------------
; Decrement, according to background_timer and foreground_timer,
; the value of background_offset and foreground_offset for 
; making the differential scroll.
;----------------------------------------------------------------------
MakeScroll:
	movem.l	a0-a1,-(a7)

	lea	bg_vars(PC),a0		;
	lea	10(a0),a1		;addr of background_timer
	subq.w	#1,(a1)			;decrement timer
	bne.s	\noscroll1		;if no zero don't scroll

	move.w	12(a0),(a1)		;else restore itimer value
	tst.w	(a0)			;test tile offset
	bne.s	\skip			;if null 
	move.w	#TILE_HEIGHT,(a0)	;get new tile
	subq.l	#5,2(a0)
\skip	subq.w	#1,(a0)


\noscroll1
	lea	4(a1),a0		;addr of fg_vars
	lea	10(a0),a1		;addr of foreground_timer
	subq.w	#1,(a1)			;decrement timer
	bne.s	\noscroll2		;if no zero don't scroll

	move.w	12(a0),(a1)		;else restore itimer value
	tst.w	(a0)			;test tile offset
	bne.s	\skip2			;if null 
	move.w	#TILE_HEIGHT,(a0)	; get new tile
	subq.l	#5,2(a0)
\skip2	subq.w	#1,(a0)
		
\noscroll2
	lea	6(a1),a0		;addr of sfg_vars
	lea	10(a0),a1		;addr of sforeground_timer
	subq.w	#1,(a1)			;decrement timer
	bne.s	\noscroll3		;if no zero don't scroll

	move.w	12(a0),(a1)		;else restore itimer value
	tst.w	(a0)			;test tile offset
	bne.s	\skip3			;if null 
	move.w	#TILE_HEIGHT,(a0)	; get new tile
	subq.l	#5,2(a0)
\skip3	subq.w	#1,(a0)

\noscroll3

	movem.l	(a7)+,a0-a1
	rts
	

;----------------------------------------------------------------------
; Show the background : no mask is used
; for tiles
;
; d0.w : offset of first line
; a0.l : plane 0 of greyscale screen
; a1.l : plane 1 of greyscale screen
; a2.l : data buffer
; a3.l : GFX buffer
;----------------------------------------------------------------------

ShowBackground:
	movem.l	d0-d4/a0-a6,-(a7)

	move.w	#128,d3			;Max lines counter
	moveq.w	#TILE_HEIGHT-1,d4	;Get vertical offset
	sub.w	d0,d4			; in 1st mosaic
	lsl.w	#3,d0			;d0*(4*2) (offset of data)
	
	movem.l	a2-a3,-(a7)		;save a2 & a3
	adda.w	d0,a3			;Addr of first mos

\New_Mosaic_Line
	moveq.w	#4,d2			;5 laps for 5 mosaics
\getmos	clr.w	d1			;Clear d1
	move.b	(a2)+,d1		;Get mosaic ID
	lsl.w	#8,d1			;d1*32*4*2
	pea	0(a3,d1.w)		;push mosaic GFX addr
	dbra.s	d2,\getmos
	movem.l	(a7)+,a2-a6		;pop all mosaics GFX addr
	addq.l	#5,(a7)			;modify a2

\cpymos
	Unmasked32Pix a6
	Unmasked32Pix a5
	Unmasked32Pix a4
	Unmasked32Pix a3
	Unmasked32Pix a2

	lea	4(a0),a0
	lea	4(a1),a1

	subq.w	#1,d3			;check if out of screen
	beq.s	\exit			
	dbra.s	d4,\cpymos		;else copy next line of mosaic

	movem.l	(a7),a2-a3		;restore saved registers
	moveq.w	#TILE_HEIGHT-1,d4	;height of mos = 32
	bra	\New_Mosaic_Line

\exit	adda.w	#8,a7
	movem.l	(a7)+,d0-d4/a0-a6
	rts



;----------------------------------------------------------------------
; Show the foreground : a greyscale mask is used 
; for each tile
;
; d0.w : offset of first line
; a0.l : plane 0 of greyscale screen
; a1.l : plane 1 of greyscale screen
; a2.l : data buffer
; a3.l : GFX buffer
;----------------------------------------------------------------------

ShowForeground:
	movem.l	d0-d4/a0-a6,-(a7)

	move.w	#128,d3			;Max lines counter
	moveq.w	#TILE_HEIGHT-1,d4	;Get vertical offset
	sub.w	d0,d4			; in 1st mosaic
	lsl.w	#4,d0			;d0*(4*4) (offset of data)
	
	movem.l	a2-a3,-(a7)		;save a2 & a3
	adda.w	d0,a3			;Addr of first mos

\New_Mosaic_Line
	moveq.w	#4,d2			;5 laps for 5 mosaics
\getmos	clr.w	d1			;Clear d1
	move.b	(a2)+,d1		;Get mosaic ID
	lsl.w	#8,d1			;d1*32*4*4
	add.w	d1,d1			;
	pea	0(a3,d1.w)		;push mosaic GFX addr
	dbra.s	d2,\getmos
	movem.l	(a7)+,a2-a6		;pop all mosaics GFX addr
	addq.l	#5,(a7)

\cpymos
	Masked32Pix	a6
	Masked32Pix	a5
	Masked32Pix	a4
	Masked32Pix	a3
	Masked32Pix	a2

	lea	4(a0),a0
	lea	4(a1),a1

	subq.w	#1,d3			;check if out of screen
	beq.s	\exit			
	dbra.s	d4,\cpymos		;else copy next line of mosaic

	movem.l	(a7),a2-a3		;restore saved registers
	moveq.w	#TILE_HEIGHT-1,d4	;height of mos = 32
	bra	\New_Mosaic_Line

\exit	adda.w	#8,a7
	movem.l	(a7)+,d0-d4/a0-a6
	rts

;----------------------------------------------------------------------
; Show the super-foreground : a greyscale mask is used 
; for each tile
;
; d0.w : offset of first line
; a0.l : plane 0 of greyscale screen
; a1.l : plane 1 of greyscale screen
; a2.l : data buffer
; a3.l : GFX buffer
;----------------------------------------------------------------------

sShowForeground:
	movem.l	d0-d4/a0-a6,-(a7)

	move.w	#128,d3			;Max lines counter
	moveq.w	#TILE_HEIGHT-1,d4	;Get vertical offset
	sub.w	d0,d4			; in 1st mosaic
	lsl.w	#4,d0			;d0*(4*4) (offset of data)
	
	movem.l	a2-a3,-(a7)		;save a2 & a3
	adda.w	d0,a3			;Addr of first mos

\New_Mosaic_Line
	moveq.w	#4,d2			;5 laps for 5 mosaics
\getmos	clr.w	d1			;Clear d1
	move.b	(a2)+,d1		;Get mosaic ID
	lsl.w	#8,d1			;d1*32*4*4
	add.w	d1,d1			;
	pea	0(a3,d1.w)		;push mosaic GFX addr
	dbra.s	d2,\getmos
	movem.l	(a7)+,a2-a6		;pop all mosaics GFX addr
	addq.l	#5,(a7)

\cpymos
	Masked32Pix	a6
	Masked32Pix	a5
	Masked32Pix	a4
	Masked32Pix	a3
	Masked32Pix	a2

	lea	4(a0),a0
	lea	4(a1),a1

	subq.w	#1,d3			;check if out of screen
	beq.s	\exit			
	dbra.s	d4,\cpymos		;else copy next line of mosaic

	movem.l	(a7),a2-a3		;restore saved registers
	moveq.w	#TILE_HEIGHT-1,d4	;height of mos = 32
	bra	\New_Mosaic_Line

\exit	adda.w	#8,a7
	movem.l	(a7)+,d0-d4/a0-a6
	rts
